function tableout = importfile(workbookFile,sheetName,startRow,endRow)
%IMPORTFILE Import data from a spreadsheet
%   DATA = IMPORTFILE(FILE) reads data from the first worksheet in the
%   Microsoft Excel spreadsheet file named FILE and returns the data as a
%   table.
%
%   DATA = IMPORTFILE(FILE,SHEET) reads from the specified worksheet.
%
%   DATA = IMPORTFILE(FILE,SHEET,STARTROW,ENDROW) reads from the specified
%   worksheet for the specified row interval(s). Specify STARTROW and
%   ENDROW as a pair of scalars or vectors of matching size for
%   dis-contiguous row intervals. To read to the end of the file specify an
%   ENDROW of inf.
%
%	Non-numeric cells are replaced with: NaN
%
% Example:
%   KWModelS1 = importfile('KW_Model.xls','KW_1',2,7162);
%
%   See also XLSREAD.

% Auto-generated by MATLAB on 2019/08/21 10:23:41

%% Input handling

% If no sheet is specified, read first sheet
if nargin == 1 || isempty(sheetName)
    sheetName = 1;
end

% If row start and end points are not specified, define defaults
if nargin <= 3
    startRow = 2;
    endRow = 7162;
end

%% Import the data, extracting spreadsheet dates in Excel serial date format
[~, ~, raw1, dates1] = xlsread(workbookFile, sheetName, sprintf('A%d:A%d',startRow(1),endRow(1)),'' , @convertSpreadsheetExcelDates);
[~, ~, raw2, dates2] = xlsread(workbookFile, sheetName, sprintf('O%d:O%d',startRow(1),endRow(1)),'' , @convertSpreadsheetExcelDates);
[~, ~, raw3, dates3] = xlsread(workbookFile, sheetName, sprintf('Q%d:Q%d',startRow(1),endRow(1)),'' , @convertSpreadsheetExcelDates);
[~, ~, raw4, dates4] = xlsread(workbookFile, sheetName, sprintf('R%d:R%d',startRow(1),endRow(1)),'' , @convertSpreadsheetExcelDates);
[~, ~, raw5, dates5] = xlsread(workbookFile, sheetName, sprintf('W%d:W%d',startRow(1),endRow(1)),'' , @convertSpreadsheetExcelDates);
raw = [raw1,raw2,raw3,raw4,raw5];
dates = [dates1,dates2,dates3,dates4,dates5];
for block=2:length(startRow)
    [~, ~, tmpRawBlock1, tmpDateNumBlock1] = xlsread(workbookFile, sheetName, sprintf('A%d:A%d',startRow(block),endRow(block)),'' , @convertSpreadsheetExcelDates);
    [~, ~, tmpRawBlock2, tmpDateNumBlock2] = xlsread(workbookFile, sheetName, sprintf('O%d:O%d',startRow(block),endRow(block)),'' , @convertSpreadsheetExcelDates);
    [~, ~, tmpRawBlock3, tmpDateNumBlock3] = xlsread(workbookFile, sheetName, sprintf('Q%d:Q%d',startRow(block),endRow(block)),'' , @convertSpreadsheetExcelDates);
    [~, ~, tmpRawBlock4, tmpDateNumBlock4] = xlsread(workbookFile, sheetName, sprintf('R%d:R%d',startRow(block),endRow(block)),'' , @convertSpreadsheetExcelDates);
    [~, ~, tmpRawBlock5, tmpDateNumBlock5] = xlsread(workbookFile, sheetName, sprintf('W%d:W%d',startRow(block),endRow(block)),'' , @convertSpreadsheetExcelDates);
    raw = [raw;tmpRawBlock1,tmpRawBlock2,tmpRawBlock3,tmpRawBlock4,tmpRawBlock5]; %#ok<AGROW>
    dates = [dates;tmpDateNumBlock1,tmpDateNumBlock2,tmpDateNumBlock3,tmpDateNumBlock4,tmpDateNumBlock5]; %#ok<AGROW>
end
raw(cellfun(@(x) ~isempty(x) && isnumeric(x) && isnan(x),raw)) = {''};
raw = raw(:,[2,3,4,5]);
dates = dates(:,1);

%% Replace non-numeric cells with NaN
R = cellfun(@(x) ~isnumeric(x) && ~islogical(x),raw); % Find non-numeric cells
raw(R) = {NaN}; % Replace non-numeric cells
R = cellfun(@(x) ~isnumeric(x) && ~islogical(x),dates); % Find non-numeric cells
dates(R) = {NaN}; % Replace non-numeric Excel dates with NaN

%% Create output variable
I = cellfun(@(x) ischar(x), raw);
raw(I) = {NaN};
data = reshape([raw{:}],size(raw));

%% Create table
tableout = table;

%% Allocate imported array to column variable names
dates(~cellfun(@(x) isnumeric(x) || islogical(x), dates)) = {NaN};
tableout.DATE = datetime([dates{:,1}].', 'ConvertFrom', 'Excel');
tableout.TP_2 = data(:,1);
tableout.TP_4 = data(:,2);
tableout.TP_5 = data(:,3);
tableout.TP_10 = data(:,4);

% For code requiring serial dates (datenum) instead of datetime, uncomment
% the following line(s) below to return the imported dates as datenum(s).

%tableout.DATE=datenum(tableout.DATE);

%% Make timetable
tableout = table2timetable(tableout);
