
set more off
clear all
clear mata
clear matrix
eststo clear

global root "C:\Users\l1pxp07\Dropbox (FRB SF)\Pascal.Paul\Current Projects\Banks\Replication Files"
global output "$root\Income Responses\Output"

**************************************************
**    		 Choose Functions & Sample 			**
**************************************************

** Dataset
global construct_dataset 	1		// Set to 1 to construct quarterly surprises

* Data Setup
global start_sample 		1994Q1 	// Choose start date for sample
global end_sample 			2007Q4  // Choose end date for sample

* Regression Specifications
global IRF_horizon 			10 		// Impulse response horizon
local j 					= 5 	// Bond maturity

**************************************************
** 					Set Up Data 				**
**************************************************

if $construct_dataset == 1 {

	cd "$root\Data\Other"

** MP Days **
	
	import excel using "MP_Days.xls", sheet("Tight") firstrow
	
	gen date2 = date(Date,"MDY")
	format date2 %td

	drop Date
	rename date2 date
	label variable date "Date"
	
	drop if date == date("9/17/2001","MDY")
	
	tempfile database
	save `database'
	
** 3-month T-Bill **
	
	import excel using "TB3.xls", sheet("TB3") clear
	
	drop C
	rename A date
	rename B TB3
	format date %td
	tsset date
	tsfill
	
	gen TB3_D = F0.TB3 - L1.TB3
	forvalues i=2/5 {
		replace TB3_D  	= F0.TB3 - L`i'.TB3 if TB3_D == .
	}
	
	replace TB3 = TB3_D
	drop TB3_D
	
	tempfile TB_3M
	save `TB_3M'
	
	use `database', clear
	merge 1:1 date using `TB_3M'
	drop if _merge == 2
	drop _merge
	
	tempfile database
	save `database'

	cd "$root\Data\Term Premia"

** Kim & Wright Term Structure Model **

	import excel using "KW_Model.xls", sheet("KW_1") firstrow clear

	format DATE %td
	rename DATE date
	tsset date
	tsfill

	drop B M X

	forvalues i=1/10 {
		rename TP_`i' KW_TP_`i'
		label variable KW_TP_`i' "Kim & Wright `i'-Year Term Premia"
	}

	drop KW_TP_6 KW_TP_7 KW_TP_8 KW_TP_9

	forvalues i=1/10 {
		rename AER_`i' KW_AER_`i'
		label variable KW_AER_`i' "Kim & Wright `i'-Year AER"
	}

	drop KW_AER_6 KW_AER_7 KW_AER_8 KW_AER_9

	forvalues i=1/10 {
		rename FY_`i' KW_FY_`i'
		label variable KW_FY_`i' "Kim & Wright `i'-Year FY"
	}

	drop KW_FY_6 KW_FY_7 KW_FY_8 KW_FY_9

	foreach x in TP FY AER {
		foreach y in 1 2 3 4 5 10 {

			gen KW_`x'_`y'_D = F0.KW_`x'_`y' - L1.KW_`x'_`y'
				forvalues i=2/5 {
					replace KW_`x'_`y'_D  	= F$numb_day_change.KW_`x'_`y' - L`i'.KW_`x'_`y' if KW_`x'_`y'_D == .
				}
		}
	}

	foreach x in TP FY AER {
		foreach y in 1 2 3 4 5 10 {
			replace KW_`x'_`y' = KW_`x'_`y'_D	
		}
	}

	drop *_D

	tempfile KW_TS
	save `KW_TS'

	use `database', clear
	merge m:1 date using `KW_TS'
	drop if _merge == 2
	drop _merge

	tempfile database
	save `database'

	drop if date == .
	drop if date == date("9/17/2001","MDY")
	
	gen date2 = qofd(date)
	format date2 %tq
	
	keep date2 *_TP_2 *_TP_5 *_TP_10 *_AER_2 *_AER_5 *_AER_10 *_FY_2 *_FY_5 *_FY_10 TB3
	rename date2 date
	
	foreach i in 2 5 10 {
		gen AER_`i'  	= KW_AER_`i'
		gen TP_`i'  	= KW_TP_`i'
		gen FY_`i' 		= KW_FY_`i'
	}
	
	collapse (sum) TP_2 TP_5 TP_10 AER_2 AER_5 AER_10 TB3, by(date)

	tempfile database
	save `database'
	
	cd "$root\Data\Other"
	
	import excel using "NIM_fred.xlsx", sheet("Sheet1") clear
	
	rename C Fred_nim 
	rename D Fred_ROA
	
	gen date = yq(A,B)
	format date %tq
	
	drop A B 
	drop if date == .
	
	tempfile Fred
	save `Fred'

	use `database', clear
	merge 1:1 date using `Fred'
	drop _merge
	
	keep if date >= quarterly("$start_sample","YQ") & date <= quarterly("$end_sample","YQ")
	
	save "$root\Income Responses\temp\impulse_data.dta", replace
	
}

use "$root\Income Responses\temp\impulse_data.dta", clear

**************************************************
** 					Local Projections 		    **
**************************************************

cd "$root\Income Responses\temp"

collapse (mean) Fred_* AER* TP* TB3, by(date)

gen nim_reg 			= .
gen net_inc_reg 		= .
gen slope 				= .
gen level 				= .

replace nim_reg 		= Fred_nim
replace net_inc_reg 	= Fred_ROA

outreg, clear
tsset date

forvalues i = 0/$IRF_horizon {
	foreach x in nim_reg net_inc_reg {
		gen `x'_lead_`i' = F`i'.`x' - L1.`x'
	}
}

foreach x in nim_reg net_inc_reg {
			
		tempname result_`x'_`j'
		postfile `result_`x'_`j'' q level level_se slope slope_se TP TP_se using result_`x'_`j'.dta, replace
		
		global temp_N 		= _N
		global laglength 	= floor(4*($temp_N/100)^(2/9))

		replace slope = AER_`j' - TB3
		replace level = TB3
		
		forvalues i = 0/$IRF_horizon {
		
			qui newey `x'_lead_`i' level slope TP_`j' `control_list', lag($laglength)
			post `result_`x'_`j'' (`i') (`=_b[level]') (`=_se[level]') (`=_b[slope]') (`=_se[slope]') (`=_b[TP]') (`=_se[TP]')
			
		}
		postclose `result_`x'_`j''
}

foreach x in nim_reg net_inc_reg {
	
		use result_`x'_`j', clear
		
			foreach m in level slope TP {
			
				rename `m' `m'_`j'_`x'
				rename `m'_se `m'_se__`j'_`x'

				gen upper_`m'_`j'_`x' 	= `m'_`j'_`x' 	+ `m'_se__`j'_`x'
				gen lower_`m'_`j'_`x' 	= `m'_`j'_`x' 	- `m'_se__`j'_`x'
				gen upper2_`m'_`j'_`x' 	= `m'_`j'_`x' 	+ 2*`m'_se__`j'_`x'
				gen lower2_`m'_`j'_`x' 	= `m'_`j'_`x' 	- 2*`m'_se__`j'_`x'

				qui graph twoway rarea upper2_`m'_`j'_`x' lower2_`m'_`j'_`x' q, color(gs14) || rarea upper_`m'_`j'_`x' lower_`m'_`j'_`x' q, color(gs12) || line `m'_`j'_`x' q, saving(result_`m'_`j'_`x', replace) legend(off) title(`m'_`x')
	}
	
	export excel using "$root\Income Responses\Matlab Figure\IRFs_matlab_agg", sheet(`x', replace) firstrow(variables)
	
}

graph combine result_level_`j'_nim_reg.gph result_slope_`j'_nim_reg.gph result_TP_`j'_nim_reg.gph result_level_`j'_net_inc_reg.gph result_slope_`j'_net_inc_reg.gph result_TP_`j'_net_inc_reg.gph

