set more off
clear all
clear mata
clear matrix
eststo clear
set maxvar 7000

global root "C:\Users\l1pxp07\Dropbox (FRB SF)\Pascal.Paul\Current Projects\Banks\Replication Files"

** Dataset
global construct_dataset 	1 	// Set to 1 to construct dataset

**************************************************
**         	 	CONSTRUCT DATASET 				**
**************************************************
*
if $construct_dataset == 1 {

	cd "$root\Data\Other"

** MP Days **
	
	import excel using "MP_Days.xls", sheet("Tight") firstrow
	
	gen date2 = date(Date,"MDY")
	format date2 %td

	drop Date
	rename date2 date
	label variable date "Date"
	
	drop if date == date("9/17/2001","MDY")
	
	tempfile database
	save `database'
	
** 3-month T-Bill **
	
	import excel using "TB3.xls", sheet("TB3") clear
	
	drop C
	rename A date
	rename B TB3
	format date %td
	tsset date
	tsfill
	
	gen TB3_D = F0.TB3 - L1.TB3
	
	if 5 >= 1 {
		forvalues i=2/5 {
			replace TB3_D  	= F0.TB3 - L`i'.TB3 if TB3_D == .
		}
	}
	
	replace TB3 = TB3_D
	
	drop TB3_D
	
	tempfile TB_3M
	save `TB_3M'
	
	use `database', clear
	merge 1:1 date using `TB_3M'
	drop if _merge == 2
	drop _merge
	
	tempfile database
	save `database'
	
** GSW-Surprises **

	import excel using "GSW_Estimates.xls", sheet("Yields") firstrow clear
	
	format Date %td
	rename Date date
	sort date
	
	gen diff_date_GSW = date - date[_n-1]
	
	tsset date
	tsfill
	
	forvalues i=1/9 {
		rename SVENY0`i' GSW_`i'
		label variable GSW_`i' "GSW `i'-Year"
	}
	rename SVENY10 GSW_10
	label variable GSW_10 "GSW 10-Year"
	
	drop GSW_6 GSW_7 GSW_8 GSW_9
	
	foreach y in 1 2 3 4 5 10 {

		gen GSW_`y'_D = F0.GSW_`y' - L1.GSW_`y'
			
		if 5 >= 1 {
			forvalues i=2/5 {
				replace GSW_`y'_D  	= F0.GSW_`y' - L`i'.GSW_`y' if GSW_`y'_D == .
			}
		}
	}
	
	foreach y in 1 2 3 4 5 10 {
	
		replace GSW_`y' = GSW_`y'_D	
		
	}

	drop *_D diff_date_GSW
	
	tempfile GSW_TS
	save `GSW_TS'
	
	use `database', clear
	merge 1:1 date using `GSW_TS'
	drop if _merge == 2
	drop _merge

	tempfile database
	save `database'
	
** Aggregate Bank Stock Price Index **

	import excel using "KBW.xls", sheet("KBW Bank Index") firstrow clear
	
	keep Date Close
	destring Close, replace force
	
	format Date %td
	rename Date date
	
	tsset date
	tsfill
	
	gen KBW = 100*(F0.Close - L1.Close)/L1.Close
			
	if 5 >= 1 {
		forvalues i=2/5 {
			replace KBW  = 100*(F0.Close - L`i'.Close)/L`i'.Close if KBW == .
		}
	}
	
	drop Close

	tempfile KBW
	save `KBW'

	use `database', clear
	merge 1:1 date using `KBW'
	drop if _merge == 2
	drop _merge

	tempfile database
	save `database'
		
** S&P 500 **

	import excel using "SP500.xls", sheet("sp500") firstrow clear
	
	keep Date Close
	
	format Date %td
	rename Date date
	
	tsset date
	tsfill
	
	gen SP500_daily = 100*(F0.Close - L1.Close)/L1.Close
			
	if 5 >= 1 {
		forvalues i=2/5 {
			replace SP500_daily  = 100*(F0.Close - L`i'.Close)/L`i'.Close if SP500_daily == .
		}
	}
	
	drop Close

	tempfile SP
	save `SP'

	use `database', clear
	merge 1:1 date using `SP'
	drop if _merge == 2
	drop _merge

	tempfile database
	save `database'
	
** S&P 500 Replication

	import excel using "SP500replication.xls", sheet("Sheet1") firstrow clear
	
	drop SP500_banks2 SP500_nonbanks2
	
	format date %td
	
	tsset date
	tsfill
	
	gen SP500_repl 				= 100*(F0.SP500_replicated - L1.SP500_replicated)/L1.SP500_replicated
	gen SP500_repl_banks 		= 100*(F0.SP500_banks1 - L1.SP500_banks1)/L1.SP500_banks1
	gen SP500_repl_nonbanks 	= 100*(F0.SP500_nonbanks1 - L1.SP500_nonbanks1)/L1.SP500_nonbanks1
	
	if 5 >= 1 {
		forvalues i=2/5 {
			replace SP500_repl  			= 100*(F0.SP500_replicated - L`i'.SP500_replicated)/L`i'.SP500_replicated if SP500_repl == .
			replace SP500_repl_banks  		= 100*(F0.SP500_banks1 - L`i'.SP500_banks1)/L`i'.SP500_banks1 if SP500_repl_bank == .
			replace SP500_repl_nonbanks  	= 100*(F0.SP500_nonbanks1 - L`i'.SP500_nonbanks1)/L`i'.SP500_nonbanks1 if SP500_repl_nonbanks == .
		}
	}

	drop SP500_replicated SP500_banks1 SP500_nonbanks1
	
	tempfile SP_repl
	save `SP_repl'
	
	use `database', clear
	merge 1:1 date using `SP_repl'
	drop if _merge == 2
	drop _merge
	
	tempfile database
	save `database'
	
	cd "$root\Data\Term Premia"

** Kim & Wright Term Structure Model **

	import excel using "KW_Model.xls", sheet("KW_1") firstrow clear
	
	format DATE %td
	rename DATE date
	tsset date
	tsfill
	
	drop B M X

	forvalues i=1/10 {
		rename TP_`i' KW_TP_`i'
		label variable KW_TP_`i' "Kim & Wright `i'-Year Term Premia"
	}

	drop KW_TP_6 KW_TP_7 KW_TP_8 KW_TP_9
	
	forvalues i=1/10 {
		rename AER_`i' KW_AER_`i'
		label variable KW_AER_`i' "Kim & Wright `i'-Year AER"
	}

	drop KW_AER_6 KW_AER_7 KW_AER_8 KW_AER_9
	
	forvalues i=1/10 {
		rename FY_`i' KW_FY_`i'
		label variable KW_FY_`i' "Kim & Wright `i'-Year FY"
	}

	drop KW_FY_6 KW_FY_7 KW_FY_8 KW_FY_9
	
	foreach x in TP FY AER {
		foreach y in 1 2 3 4 5 10 {

			gen KW_`x'_`y'_D = F0.KW_`x'_`y' - L1.KW_`x'_`y'
			
			if 5 >= 1 {
				forvalues i=2/5 {
					replace KW_`x'_`y'_D  	= F0.KW_`x'_`y' - L`i'.KW_`x'_`y' if KW_`x'_`y'_D == .
				}
			}
		}
	}
	
	foreach x in TP FY AER {
		foreach y in 1 2 3 4 5 10 {
			replace KW_`x'_`y' = KW_`x'_`y'_D	
		}
	}

	drop *_D
	
	tempfile KW_TS
	save `KW_TS'

	use `database', clear
	merge m:1 date using `KW_TS'
	drop if _merge == 2
	drop _merge
	
	save "$root\Stock Price Responses\Output\Bank_level_data.dta", replace
	
}

use "$root\Stock Price Responses\Output\Bank_level_data.dta", clear

foreach i in 1 2 3 4 5 10 {
	gen AER_`i'  	= KW_AER_`i'
	gen TP_`i'  	= KW_TP_`i'
	gen FY_`i' 		= KW_FY_`i'
}

gen SR = TB3

order date* KBW SP500* GSW* KW_*

cd "$root\Stock Price Responses\Tables"

gen slope 					= .
gen TP 						= .
gen LR 						= .
gen AER 					= .
gen sample  				= .
gen bank 					= .
gen meeting 				= .
gen mat_var_adj 			= .
gen control     			= .
gen control2    			= .
gen interact 				= .

*********************************
** Run Aggregate Stock Indices **
*********************************
			
keep if date>=date("$start_sample","MDY") & date<=date("$end_sample","MDY")
	
foreach y in KBW SP500_repl_nonbanks {

	foreach i in 2 5 10 {
	
		replace slope 	= GSW_`i'- SR
			
		eststo: qui reg `y' SR slope, vce(robust)
		qui outreg, merge(reg1) bdec(2) keep(SR slope) se starloc(1) starlevels(10 5 1) rtitle("\Delta Y^{3M}_t" \ "" \ "\Delta(Y^m_t - Y^{3M}_t)") summtitles("R^2" \ "Meetings")
		
	}
	qui outreg, replay(reg1) addc("")			
		
}
qui outreg, replay append(reg1)
outreg using "Table_4_1.tex", replay starlevels(10 5 1) starloc(1) ///
	ctitle("", "", "\underline{ KBW Index }", "", "", "", "\underline{ S\&P 500 }" , "" \ "m = ", "2-year", "5-year", "10-year", "", "2-year", "5-year", "10-year") ///
	hlines(101000101) spaceb(00100) spacea(0000010) spaceht(3) ///
	tex replace
	eststo clear
	outreg, clear

foreach y in KBW SP500_repl_nonbanks /*SP500_repl_banks SP500_daily*/ {

	foreach i in 2 5 10 {
	
		replace AER		= AER_`i' - SR
		replace TP 		= TP_`i'
		
		eststo: qui reg `y' SR AER TP, vce(robust)
		qui outreg, merge(reg2) bdec(2) keep(SR AER TP) se starloc(1) starlevels(10 5 1) rtitle("\Delta Y^{3M}_t" \ "" \ "\Delta(EH^m_t-Y^{3M}_t)" \ "" \ "\Delta TP^m_t") summtitles("R^2" \ "Meetings")
		
	}
	qui outreg, replay(reg2) addc("")			

}
qui outreg, replay append(reg2)
outreg using "Table_4_2.tex", replay starlevels(10 5 1) starloc(1) ///
	ctitle("", "", "\underline{ KBW Index }", "", "", "", "\underline{ S\&P 500 }" , "" \ "m = ", "2-year", "5-year", "10-year", "", "2-year", "5-year", "10-year") ///
	hlines(10100000101) spaceb(00100) spacea(000000010) spaceht(3) ///
	tex replace
	eststo clear
	outreg, clear
	

